print("loading insync caja plugin..")  # noqa
import json
import os
import socket
import sys

try:
    from urllib import unquote  # Python 2.X
except ImportError:
    from urllib.parse import unquote  # Python 3+

try:
    from importlib import reload  # Python 3.4+
except ImportError:
    from imp import reload  # Python 2.x-3.3

import gi  # noqa isort:skip

gi.require_version("Caja", "2.0")  # noqa isort:skip

from gi.repository import Caja, GObject  # noqa isort:skip

DEV_MODE = False

# http://www.gtk.org/tutorial1.2/gtk_tut-13.html
# http://pygtk.org/docs/pygtk/class-gtkaction.html
# gtk.Action(name, label, tooltip, stock_id)

if sys.getdefaultencoding() != "utf-8":
    reload(sys)  # noqa
    sys.setdefaultencoding("utf-8")  # pylint: disable=E1101


def ipc_insync(**kw):
    data = json.dumps(kw)
    socket_file = u"/tmp/insync%r.sock" % os.getuid()
    if not os.path.exists(socket_file):
        if DEV_MODE:
            print("WARN: insync socket not found")
        return None
    sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    sock.settimeout(0.1)
    try:
        sock.connect(socket_file)
        sock.send(str.encode(data))
        res = sock.recv(4096)
        sock.close()
    except Exception:
        return None
    if res.strip():
        return json.loads(res.strip())
    else:
        return None


_EMBLEM_KEYS = {
    "SYNCED": "emblem-insync-synced",
    "SYNCING": "emblem-insync-syncing",
    "ERROR": "emblem-insync-error",
}


class InsyncPlugin(GObject.GObject, Caja.MenuProvider, Caja.InfoProvider):

    # Caja crashes if a plugin doesn't implement the __init__ method.
    # See Bug #374958
    def __init__(self):
        self.file_list = dict()

    def get_background_items(self, window, file):
        return None

    def get_file_items(self, window, files):
        if DEV_MODE:
            print(("get file items", files))

        if len(files) != 1:
            # TODO what can we do with mutiple files here
            return None

        file = files[0]
        if file.get_uri_scheme() != "file":
            return None

        path = os.path.realpath(unquote(file.get_uri()[len("file://") :]))
        if sys.version_info[0] < 3:
            path = unicode(path)  # noqa
        else:
            path = str(path)
        if DEV_MODE:
            print(path)
        is_dir = os.path.isdir(path)

        cm_info = ipc_insync(command="CONTEXT-MENU-ITEMS", full_path=path)

        if cm_info:
            title, cm_items = cm_info
            tip = "Insync folder actions" if is_dir else "Insync file actions"
            item = Caja.MenuItem(
                name="Insync:" + title, label=title, tip=tip, icon="insync"
            )
            sub_menu = Caja.Menu()
            item.set_submenu(sub_menu)

            item_id = 0
            for text, cmd in cm_items:
                if text == "separator":
                    text = u"\u2015" * 10
                    name = "Insync:separator:" + str(item_id)
                else:
                    name = "Insync::%d" % (item_id)
                item_id += 1
                menu_item = Caja.MenuItem(name=name, label=text, tip=text)
                if cmd:
                    menu_item.connect("activate", self.do_action, file, cmd)
                else:
                    menu_item.sensitive = False
                sub_menu.append_item(menu_item)

            return [item]
        return None

    def do_action(self, menu, file, method):
        path = unquote(file.get_uri()[len("file://") :])
        ipc_insync(method=method, full_path=path)

    def update_file_info(self, file):
        if file.get_uri_scheme() != "file":
            return

        filename = unquote(file.get_uri()[len("file://") :])
        filename = os.path.realpath(filename)
        status = ipc_insync(command="GET-FILE-STATUS", full_path=filename)
        if DEV_MODE:
            print(filename, status)
        if status and status != "UNKNOWN":
            emblem = _EMBLEM_KEYS[status]
            if os.path.isdir(filename):
                is_shared = ipc_insync(command="IS-FILE-SHARED", full_path=filename)
                if is_shared:
                    emblem += "-shared"
            file.add_emblem(emblem)
            old_emblem = self.file_list.get(filename)
            self.file_list[filename] = emblem
            if old_emblem != emblem:
                file.invalidate_extension_info()
        elif self.file_list.pop(filename, None):
            file.invalidate_extension_info()
